@@ -329,6 +329,36 @@ char* gtk_dialog_gettext_big(const char* question)
  return tentry;
 }
 
+char* gtk_dialog_getpass(const char* question)
+{
+ GtkWidget* dialog, *label, *button, *entry;
+ debug("%s:%d gtk_dialog_gettext",__FILE__,__LINE__);
+ if (!question) warn("question=NULL: %s:%d",__FILE__,__LINE__);
+ strcpy(tentry, "");
+ dialog = gtk_dialog_new();
+ gtk_window_set_title(GTK_WINDOW(dialog), "Enter text");
+ gtk_widget_set_usize(dialog, 300, 100);
+ gtk_window_set_modal(GTK_WINDOW(dialog), TRUE);
+ label = gtk_label_new(question);
+ entry = gtk_entry_new();
+ gtk_entry_set_max_length(GTK_ENTRY(entry), MAX_ENTRY_LENGTH-2);
+ gtk_entry_set_visibility(GTK_ENTRY(entry), FALSE);
+ gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), label, TRUE, TRUE, 0);
+ gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), entry, TRUE, TRUE, 0);
+ gtk_widget_show(label);
+ gtk_widget_show(entry);
+ gtk_widget_grab_focus(entry);
+ button = gtk_button_new_with_label("OK");
+ gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area), button, TRUE, TRUE, 0);
+ gtk_signal_connect(GTK_OBJECT(dialog), "delete_event", GTK_SIGNAL_FUNC(delete_event_proc), (gpointer)dialog);
+ gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(ok_proc), (gpointer)dialog);
+ gtk_signal_connect(GTK_OBJECT(entry), "activate", GTK_SIGNAL_FUNC(entry_proc), (gpointer)dialog);
+ gtk_signal_connect(GTK_OBJECT(entry), "changed", GTK_SIGNAL_FUNC(entry_changed_proc), (gpointer)dialog);
+ gtk_widget_show(button);
+ gtk_widget_show(dialog);
+ gtk_main();
+ return tentry;
+}
 
 char* gtk_dialog_gettext(const char* question)
 {
