@@ -31,6 +31,7 @@ extern struct mem_list* mlist;
 char** autostr=0;
 int    nautostr=0;
 int*   autoidx=0;
+char*  pass=0;
 
 void disconnect()
 {
@@ -66,7 +67,6 @@ void qexec_quit( GtkWidget *w,gpointer data)
  query_exec = NULL;
 }
 
-
 int menu_quit(GtkWidget *w, gpointer data)
 {
  debug("%s:%d menu_quit",__FILE__,__LINE__);
@@ -87,6 +87,17 @@ int menu_quit(GtkWidget *w, gpointer data)
  return TRUE;
 }
 
+int can_connect(char* templ)
+{
+ char errstr[MAX_ENTRY_LENGTH];
+ debug("%s:%d can_connect",__FILE__,__LINE__);
+ connection = gtk_connect_db(templ, pass, errstr);
+ if (!connection) return 0;
+ connected=1;
+ disconnect();
+ return 1;
+}
+
 
 void select_child_main_tree(GtkWidget* w, gpointer data)	/* s, d, c */
 {
@@ -363,7 +374,7 @@ void refresh_all()
  shred_memlist();
  if (currentbase) free(currentbase);
  currentbase=NULL;
- connection = gtk_connect_db(basen, errstr);
+ connection = gtk_connect_db(basen, pass, errstr);
  if (!connection)
      {
       fatal_nodb(basen,errstr,0);
@@ -380,7 +391,6 @@ void refresh_all()
      }
 }
 
-
 void cmenu_response(GtkWidget* w, gpointer data)
 {
  debug("%s:%d cmenu_response",__FILE__,__LINE__);	/* ON SELTYPE t one of tables, T All tables (node in tree) */
@@ -412,6 +422,11 @@ void set_wname_tab(char* arg)
  gtk_window_set_title (GTK_WINDOW(main_wnd), name);
 }
 
+void refresh_tab_view(GtkWidget* w, gpointer data)
+{
+ destruct_tab_view(w,data);
+ browse_table(components.selected);
+}
 
 void destruct_tab_view(GtkWidget* w, gpointer data)
 {
@@ -423,6 +438,10 @@ void destruct_tab_view(GtkWidget* w, gpointer data)
  components.button_save = NULL;
  if (components.button_destroy) destroy_widget(components.button_destroy);
  components.button_destroy = NULL;
+ if (components.button_refresh) destroy_widget(components.button_refresh);
+ components.button_refresh = NULL;
+ if (components.button_update) destroy_widget(components.button_update);
+ components.button_update = NULL;
  rows = components.rows;
  cols = components.cols;
  components.last_selected = NULL;
@@ -476,6 +495,85 @@ void drop_table(char* tabname)
  gtk_dialog_printf("Table %s dropped", tabname);
 }
 
+int numeric(char* val)
+{
+ int i;
+ float v;
+ if (!val) return 0;
+ i = sscanf(val,"%f", &v);
+ if (i==1) return 1;
+ else return 0;
+}
+
+void update_database(GtkWidget* w, gpointer data)
+{
+ int i,j;
+ char*** ptr;
+ int row,col;
+ char errstr[MAX_QUERY_LENGTH+100];
+ char* gent;
+ execute_printf(connection, &ptr, &row, &col, errstr, 1, "SELECT * FROM %s LIMIT 1", components.selected);
+ if (row<0 || col<0)
+   {
+    fatal_here(HERE, "get column name failed");
+    return;
+   }
+ for (i=1;i<components.rows;i++) 
+ for (j=0;j<components.cols;j++)
+   {
+    gent = gtk_entry_get_text(GTK_ENTRY(components.table_entry[i][j]));
+    if (strcmp(gent,components.data[i][j])) changed_field(components.table_entry[i][j],(void*)((i<<0x10)+j));
+   }
+ free_p3c(&ptr, row, col);
+}
+
+void changed_field(GtkWidget* field, gpointer which)
+{
+ char*** ptr;
+ int row,col;
+ char errstr[MAX_QUERY_LENGTH+100];
+ char querystr[STDSTRING];
+ char colname[STDSTRING];
+ char* gent;
+ int i,j;
+ int nmrc;
+ j = (int)which & 0xFFFF;
+ i = ((int)which & 0xFFFF0000)>>0x10;
+ execute_printf(connection, &ptr, &row, &col, errstr, 1, "SELECT * FROM %s LIMIT 1", components.selected);
+ if (row<0 || col<0)
+   {
+    fatal_here(HERE, "get column name failed");
+    return;
+   }
+ strcpy(colname, ptr[0][j]);
+ free_p3c(&ptr, row, col);
+ printf("changed field: %d, %d %s:%s\n", i, j, colname,components.data[i][j]);
+ gent = gtk_entry_get_text(GTK_ENTRY(field));
+ if (!gent) fatal_here(HERE, "text is null"); 
+ /*if (!strcmp(gent,"")) */
+ if (numeric(components.data[i][j])) nmrc = 1;
+ else nmrc = 0;
+ if (!nmrc) sprintf(querystr, "UPDATE %s SET %s='%s' WHERE %s='%s'", components.selected, colname, gent, colname, components.data[i][j]);
+ else sprintf(querystr, "UPDATE %s SET %s=%s WHERE %s=%s", components.selected, colname, gent, colname, components.data[i][j]);
+ printf("%s\n", querystr);
+ execute_printf(connection, &ptr, &row, &col, errstr, 0, querystr);
+ if (row<0 || col<0)
+   {
+    printf("DBMS warning: %s\n", errstr);
+    gtk_entry_set_text(GTK_ENTRY(field), components.data[i][j]);
+    return;
+   }
+ free_p3c(&ptr, row, col);
+ free(components.data[i][j]);
+ components.data[i][j] = 0;
+#ifdef SMALLOC
+ components.data[i][j] = (char*)dmmalloc(strlen(gent)+1, HERE);
+#else
+ components.data[i][j] = (char*)mmalloc(strlen(gent)+1);
+#endif
+ if (!components.data[i][j]) fatal_here(HERE,"malloc failed");
+ strcpy(components.data[i][j], gent);
+}
 
 void browse_table(char* tabname)
 {
@@ -534,7 +632,8 @@ void browse_table(char* tabname)
     components.table_entry[i][j] = gtk_entry_new();						/*XXX there were malloc error */
     if (i) gtk_entry_set_text(GTK_ENTRY(components.table_entry[i][j]), ptr[i+rrow][j+rcol]);
     else   gtk_entry_set_text(GTK_ENTRY(components.table_entry[i][j]), ptr[0][j+rcol]);
-    gtk_entry_set_editable(GTK_ENTRY(components.table_entry[i][j]), FALSE);
+    if (!i) gtk_entry_set_editable(GTK_ENTRY(components.table_entry[i][j]), FALSE);
+    else gtk_signal_connect(GTK_OBJECT(components.table_entry[i][j]),"activate", GTK_SIGNAL_FUNC(changed_field), (void*)((i<<0x10)+j));
     gtk_table_attach(GTK_TABLE (components.table_view), components.table_entry[i][j], j, j+1, i, i+1,GTK_FILL,GTK_FILL, 0, 0);
     gtk_widget_show(components.table_entry[i][j]);
     if (!cont) { gtk_dialog_printf("Interrupted!"); goto out_loop; }
@@ -550,14 +649,27 @@ void browse_table(char* tabname)
  gtk_widget_show(components.scroller_view);
  gtk_table_attach(GTK_TABLE(components.table), components.scroller_view, 20, 80, 0, 77, 	/* CAN BE CHANGED */
 		 GTK_FILL|GTK_EXPAND,GTK_FILL|GTK_EXPAND, 0, 0);
+ 
  components.button_destroy = gtk_button_new_with_label("Destroy");
  gtk_signal_connect(GTK_OBJECT(components.button_destroy),"clicked", GTK_SIGNAL_FUNC(destruct_tab_view), NULL);
- gtk_table_attach (GTK_TABLE(components.table), components.button_destroy, 20, 50, 78, 80,GTK_FILL|GTK_SHRINK,GTK_FILL|GTK_SHRINK, 0, 0);
+ gtk_table_attach (GTK_TABLE(components.table), components.button_destroy, 20, 35, 78, 80,GTK_FILL|GTK_SHRINK,GTK_FILL|GTK_SHRINK, 0, 0);
  gtk_widget_show(components.button_destroy);
+ 
  components.button_save = gtk_button_new_with_label("Save");
  gtk_signal_connect(GTK_OBJECT(components.button_save),"clicked", GTK_SIGNAL_FUNC(save_results_mainwnd), NULL);
- gtk_table_attach (GTK_TABLE (components.table), components.button_save, 50, 80, 78, 80,GTK_FILL|GTK_SHRINK,GTK_FILL|GTK_SHRINK, 0, 0);
+ gtk_table_attach (GTK_TABLE (components.table), components.button_save, 35, 50, 78, 80,GTK_FILL|GTK_SHRINK,GTK_FILL|GTK_SHRINK, 0, 0);
  gtk_widget_show(components.button_save);
+ 
+ components.button_refresh = gtk_button_new_with_label("Refresh");
+ gtk_signal_connect(GTK_OBJECT(components.button_refresh),"clicked", GTK_SIGNAL_FUNC(refresh_tab_view), NULL);
+ gtk_table_attach (GTK_TABLE (components.table), components.button_refresh, 50, 65, 78, 80,GTK_FILL|GTK_SHRINK,GTK_FILL|GTK_SHRINK, 0, 0);
+ gtk_widget_show(components.button_refresh);
+ 
+ components.button_update = gtk_button_new_with_label("Update");
+ gtk_signal_connect(GTK_OBJECT(components.button_update),"clicked", GTK_SIGNAL_FUNC(update_database), NULL);
+ gtk_table_attach (GTK_TABLE (components.table), components.button_update, 65, 80, 78, 80,GTK_FILL|GTK_SHRINK,GTK_FILL|GTK_SHRINK, 0, 0);
+ gtk_widget_show(components.button_update);
+ 
  set_wname_tab(tabname);
 }
 
@@ -825,6 +937,15 @@ void copy_database(GtkWidget* w, gpointer data)
  char*** ptr;
  int row,col;
  debug("%s:%d copy_database",__FILE__,__LINE__);
+ if (pass) 
+   { 
+    gtk_dialog_printf("This option is currently not implemented when\n"
+		    "password is used, beause cannot automatically pass\n"
+		    "password to psql, try removing 'password' option from\n"
+		    "pg_hba.conf and replacing it to 'trust', if You are\n"
+		    "a DB admin, else You probably cannot create DB anyway\n");
+    return;
+   }
  emerg = !connected;
  if (!connected && !dummy_connect(1,TEMPLATE))
     {
@@ -857,7 +978,7 @@ void copy_database(GtkWidget* w, gpointer data)
  free_p3c(&ptr, row, col);
  system_printf("pg_dump -i %s | psql %s", dbs,db2);
  if (!emerg) return;
- connection = gtk_connect_db(db2, errstr);
+ connection = gtk_connect_db(db2, pass, errstr);
  if (!connection)
      {
       fatal_nodb(db2,errstr,0);
@@ -885,6 +1006,15 @@ void create_DB_from_file(GtkWidget* w, gpointer data)
  int l,row,col,emerg;
  char winname[STDSTRING];
  debug("%s:%d create_DB_from_file",__FILE__,__LINE__);
+ if (pass) 
+   { 
+    gtk_dialog_printf("This option is currently not implemented when\n"
+		    "password is used, beause cannot automatically pass\n"
+		    "password to psql, try removing 'password' option from\n"
+		    "pg_hba.conf and replacing it to 'trust', if You are\n"
+		    "a DB admin, else You probably cannot create DB anyway\n");
+    return;
+   }
  dbn = gtk_dialog_gettext_printf("Need DB name to fill from a SQL file:");
  if (!strcmp(dbn,""))
     {
@@ -937,7 +1067,7 @@ void create_DB_from_file(GtkWidget* w, gpointer data)
       sprintf(winname, "PgXexplorer: (no database opened currently)");
       gtk_window_set_title (GTK_WINDOW(main_wnd), winname);
      }
-   connection = gtk_connect_db(dbname, errstr);
+   connection = gtk_connect_db(dbname, pass, errstr);
    if (!connection)
      {
       fatal_nodb(dbname,errstr,0);
@@ -964,6 +1094,15 @@ void save_database(GtkWidget* w, gpointer data)
  char dbs[STDSTRING];
  int emerg;
  debug("%s:%d save_database",__FILE__,__LINE__);
+ if (pass) 
+   { 
+    gtk_dialog_printf("This option is currently not implemented when\n"
+		    "password is used, beause cannot automatically pass\n"
+		    "password to psql, try removing 'password' option from\n"
+		    "pg_hba.conf and replacing it to 'trust', if You are\n"
+		    "a DB admin, else You probably cannot create DB anyway\n");
+    return;
+   }
  if (!connected) a=0;
  else
  a = gtk_dialog_with_2_buttons("Do You want to save Current database or Other database?", "Current", "Other");
@@ -1053,7 +1192,7 @@ void reload_bases(char* db, int ask)
    }*/
  sprintf(winname, "PgXexplorer: (no database opened currently)");
  gtk_window_set_title (GTK_WINDOW(main_wnd), winname);
- connection = gtk_connect_db(dbname, errstr);
+ connection = gtk_connect_db(dbname, pass, errstr);
  if (!connection)
    {
     fatal_nodb(dbname,errstr,0);
@@ -1104,7 +1243,7 @@ void load_database(GtkWidget* w, gpointer data)
     sprintf(winname, "PgXexplorer: (no database opened currently)");
     gtk_window_set_title (GTK_WINDOW(main_wnd), winname);
    }
- connection = gtk_connect_db(dbname, errstr);
+ connection = gtk_connect_db(dbname, pass, errstr);
  if (!connection)
      {
       fatal_nodb(dbname,errstr,0);
@@ -1211,7 +1350,7 @@ void create_database(GtkWidget* w, gpointer data)
       sprintf(winname, "PgXexplorer: (no database opened currently)");
       gtk_window_set_title (GTK_WINDOW(main_wnd), winname);
      }
-   connection = gtk_connect_db(dbname, errstr);
+   connection = gtk_connect_db(dbname, pass, errstr);
    if (!connection)
      {
       fatal_nodb(dbname,errstr,0);
@@ -1848,6 +1987,7 @@ GtkWidget* create_global_autocomplete_cmenu()
  gtk_widget_show(mi);
  gtk_menu_append(GTK_MENU(cm), mi);
  tab = gtk_table_new(nautostr, 1, FALSE);
+ qsort(autostr, nautostr, sizeof(char*), (int(*)(const void*, const void*))strcmp);
  for (i=0;i<nautostr;i++)
    {
     bt = gtk_button_new_with_label(autostr[i]);
@@ -1935,7 +2075,11 @@ void process_parser(GtkWidget* w)
  debug("%s:%d process_parser",HERE);
  debug("PROCESS_PARSER");
  len = gtk_text_get_length(GTK_TEXT(w));
- if (len<=1) return;
+ if (len<=1) 
+   {
+    destroy_global_auto_complete();
+    return;
+   }
  if (!w) { debug("%s:%d, GTK_TEXT is NULL", HERE); return; }
  text = malloc(len+1);
  for (i=0;i<len;i++) text[i] = GTK_TEXT_INDEX(GTK_TEXT(w), i);
@@ -2046,7 +2190,7 @@ void query_executor(GtkWidget* w, gpointer data)
  gtk_signal_connect(GTK_OBJECT(button[0]), "clicked", GTK_SIGNAL_FUNC(execute_proc), (gpointer)(text));
  gtk_signal_connect(GTK_OBJECT(button[1]), "clicked", GTK_SIGNAL_FUNC(save_proc), (gpointer)(text));
  gtk_signal_connect(GTK_OBJECT(button[2]), "clicked", GTK_SIGNAL_FUNC(load_proc), (gpointer)(text));
- gtk_window_set_title(GTK_WINDOW(window), "SQL Query Editor, type `;' to execute query");
+ gtk_window_set_title(GTK_WINDOW(window), "SQLExec, `;' to execute, 'TAB' to autocomplete");
  gtk_widget_show(window);
  query_exec = window;
  gtk_main();
@@ -2099,7 +2243,7 @@ int dummy_connect(int cn, char* templ)
  debug("%s:%d dummy_connect",__FILE__,__LINE__);
  if (cn==1)
   {
-   connection = gtk_connect_db(templ, errstr);
+   connection = gtk_connect_db(templ, pass, errstr);
    if (!connection)
      {
       gtk_dialog_printf("Connect to template DB: %s failed\nDBMS returned:\n%s", templ,errstr);
@@ -2122,12 +2266,15 @@ int parse_cmdline(int lb, char** par, GtkWidget* w)
  debug("%s:%d parse_cmdline",__FILE__,__LINE__);
  gotdb=0;
  dbname = NULL;
- while ((ch = getopt(lb,par,"d:s:p:b:t:m:hev")) != -1)
+ while ((ch = getopt(lb,par,"d:s:p:b:t:m:P:hev")) != -1)
    {
     switch (ch)
       {
        case 's': setenv("PGHOST", optarg, 1); break;
        case 'p': setenv("PGPORT", optarg, 1); break;
+       case 'P': pass = malloc(strlen(optarg)+1); 
+		 if (!pass) fatal_here(HERE, "malloc");
+		 strcpy(pass, optarg); break;
        case 'b': setenv("PGDATA", optarg, 1); break;
        case 't': tmp=atoi(optarg); if (tmp<=0) fatal("timeout value: %d too small", tmp); timeout=tmp; break;
        case 'd': gotdb=1; dbname = malloc(strlen(optarg)+1); strcpy(dbname, optarg); break;
@@ -2140,6 +2287,7 @@ int parse_cmdline(int lb, char** par, GtkWidget* w)
 				 "-h this help\n"
 				 "-s server name\n"
 				 "-p port\n"
+				 "-P password\n"
 				 "-b base dir\n"
 				 "-t timeout connection\n"
 				 "-m max_tables in EntireDB\n"
@@ -2150,6 +2298,18 @@ int parse_cmdline(int lb, char** par, GtkWidget* w)
 				   "-h help"); return 1;
       }
    }
+ if (!pass)
+   {
+    char* tmp;
+    if (!can_connect(TEMPLATE))
+     {
+      tmp = gtk_dialog_getpass("ENTER PASSWORD:");
+      if (!tmp) fatal_here(HERE, "tmp is NULL");
+      pass = malloc(strlen(tmp)+1);
+      if (!pass) fatal_here(HERE, "cannot malloc pass");
+      strcpy(pass, tmp);
+     }
+   }
  if (!gotdb)
     {
      if (dummy_connect(1,TEMPLATE))
@@ -2164,7 +2324,7 @@ int parse_cmdline(int lb, char** par, GtkWidget* w)
      gtk_dialog_printf("Empty Database name, cannot be opened.\nSystem will now exit.");
      return 1;
     }
- connection = gtk_connect_db(dbname, errstr);
+ connection = gtk_connect_db(dbname, pass, errstr);
  if (!connection)
    {
     fatal_nodb(dbname,errstr,1);
@@ -2185,7 +2345,6 @@ int parse_cmdline(int lb, char** par, GtkWidget* w)
 
 void todo()
 {
- error("in mmsystem.c");
  printf("If there is NO TABLES/TRIGGERS/VIEWS exception, create table option\n");
  printf("very carefully check dmem, struct dmem_list - ALL FUNCTIONS\n");
  printf("dmem rather checked, but m... *NOT*\n");
@@ -2200,7 +2359,8 @@ void todo()
  printf("na ktorej tablei trigger - p270, 277,w C p288, READ CONTRIB\n");
  printf("read pgsql_book!\n");
  printf("pg_proc - proname, IN TREE (NO!-too much), BUT CAN BE CHOOSEN IN TRIG_WIZ\n");
- printf("\n");
+ printf("DB_COPY, CREATE_FROM_FILE ETC when password used.\n");
+ printf("VISUAL_EDIT>> ADD ROWS, DROP ROWS\n");
 }
 
 
@@ -2253,6 +2413,8 @@ int main(int lb, char **par)
      gtk_window_set_title (GTK_WINDOW(window), winname);
     }
   gtk_main();
+  if (pass) free(pass);
+  pass = NULL;
   return(0);
 }
 
