@@ -32,6 +32,8 @@ struct main_components
  GtkWidget* scroller_view;
  GtkWidget* button_save;
  GtkWidget* button_destroy;
+ GtkWidget* button_refresh;
+ GtkWidget* button_update;
  GtkWidget*** table_entry;
  GtkWidget* last_selected;
  int ntables,nviews,ntriggers,cols,rows,acols,arows,rrow,rcol;
